/*! \file
 *  \brief Заголовочный файл c оптимизированными функциями библиотеки
 *  \copyright 2018-2024 RnD Center ELVEES, JSC
 *  \author Фролов Андрей
 */

#ifndef _ASM_FUNC_H
#define _ASM_FUNC_H

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

/*!
 *  \fn void adds16(int16_t *src0, int16_t *src1, int16_t *dst, int32_t size)
 *  \brief Сумма элементов типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] src1 входные данные типа int16
 *  \param[out] dst выходные данные типа int16
 *  \param[in] size размер входных данных
 */
extern void adds16(int16_t *src0, int16_t *src1, int16_t *dst, int32_t size);
/*!
 *  \fn void adds32(int32_t *src0, int32_t *src1, int32_t *dst, int32_t size)
 *  \brief Сумма элементов типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[out] dst выходные данные типа int32
 *  \param[in] size размер входных данных
 */
extern void adds32(int32_t *src0, int32_t *src1, int32_t *dst, int32_t size);
/*!
 *  \fn void add_fl(float *src0, float *src1, float *dst, int32_t size)
 *  \brief Сумма элементов типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] src1 входные данные типа float
 *  \param[out] dst выходные данные типа float
 *  \param[in] size размер входных данных
 */
extern void add_fl(float *src0, float *src1, float *dst, int32_t size);
/*!
 *  \fn void add_db(double *src0, double *src1, double *dst, int32_t size)
 *  \brief Сумма элементов типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] src1 входные данные типа double
 *  \param[out] dst выходные данные типа double
 *  \param[in] size размер входных данных
 */
extern void add_db(double *src0, double *src1, double *dst, int32_t size);

/*!
 *  \fn int32_t dotprod(int16_t *src0, int16_t *src1, int32_t size)
 *  \brief Скалярное произведение векторов типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] src1 входные данные типа int16
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32 результат скалярного произведения
 */
extern int32_t dotprod(int16_t *src0, int16_t *src1, int32_t size);
/*!
 *  \fn int64_t dotprod32(int32_t *src0, int32_t *src1, int32_t size)
 *  \brief Скалярное произведение векторов типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int64 результат скалярного произведения
 */
extern int64_t dotprod32(int32_t *src0, int32_t *src1, int32_t size);
/*!
 *  \fn float dotprod_fl(float *src0, float *src1, int32_t size)
 *  \brief Скалярное произведение векторов типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] src1 входные данные типа float
 *  \param[in] size размер входных данных
 *  \return
 *      - \b float результат скалярного произведения
 */
extern float dotprod_fl(float *src0, float *src1, int32_t size);
/*!
 *  \fn double dotprod_db(double *src0, double *src1, int32_t size)
 *  \brief Скалярное произведение векторов типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] src1 входные данные типа double
 *  \param[in] size размер входных данных
 *  \return
 *      - \b double результат скалярного произведения
 */
extern double dotprod_db(double *src0, double *src1, int32_t size);

/*!
 *  \fn int64_t vecsumsq(int16_t *src0, int32_t size)
 *  \brief Корень из суммы элементов вектора типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32 корень из суммы элементов вектора
 */
extern int64_t vecsumsq(int16_t *src0, int32_t size);
/*!
 *  \fn int64_t vecsumsq32(int32_t *src0, int32_t size)
 *  \brief Корень из суммы элементов вектора типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int64 корень из суммы элементов вектора
 */
extern int64_t vecsumsq32(int32_t *src0, int32_t size);
/*!
 *  \fn float vecsumsq_fl(float *src0, int32_t size)
 *  \brief Корень из суммы элементов вектора типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] size размер входных данных
 *  \return
 *      - \b float корень из суммы элементов вектора
 */
extern float vecsumsq_fl(float *src0, int32_t size);
/*!
 *  \fn double vecsumsq_db(double *src0, int32_t size)
 *  \brief Корень из суммы элементов вектора типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] size размер входных данных
 *  \return
 *      - \b double корень из суммы элементов вектора
 */
extern double vecsumsq_db(double *src0, int32_t size);

/*!
 *  \fn void neg32(int32_t *src0, int32_t *dst, int32_t size)
 *  \brief Отрицание элементов вектора типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[out] dst выходные данные типа int32
 *  \param[in] size размер входных данных
 */
extern void neg32(int32_t *src0, int32_t *dst, int32_t size);
/*!
 *  \fn void neg16(int16_t *src0, int16_t *dst, int32_t size)
 *  \brief Отрицание элементов вектора типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[out] dst выходные данные типа int16
 *  \param[in] size размер входных данных
 */
extern void neg16(int16_t *src0, int16_t *dst, int32_t size);
/*!
 *  \fn void neg_fl(float *src0, float *dst, int32_t size)
 *  \brief Отрицание элементов вектора типа float
 *  \param[in] src0 входные данные типа float
 *  \param[out] dst выходные данные типа float
 *  \param[in] size размер входных данных
 */
extern void neg_fl(float *src0, float *dst, int32_t size);
/*!
 *  \fn void neg_db(double *src0, double *dst, int32_t size)
 *  \brief Отрицание элементов вектора типа double
 *  \param[in] src0 входные данные типа double
 *  \param[out] dst выходные данные типа double
 *  \param[in] size размер входных данных
 */
extern void neg_db(double *src0, double *dst, int32_t size);

/*!
 *  \fn void w_vec(int16_t *src0, int16_t *src1, int16_t w, int16_t *dst, int32_t size)
 *  \brief Сложение элементов вектора с взвешенными элементами другого вектора для типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] src1 входные данные типа int16
 *  \param[in] w весовой коэффициент
 *  \param[out] dst выходные данные типа int16
 *  \param[in] size размер входных данных
 */
extern void w_vec(int16_t *src0, int16_t *src1, int16_t w, int16_t *dst, int32_t size);
/*!
 *  \fn void w_vec32(int32_t *src0, int32_t *src1, int32_t w, int32_t *dst, int32_t size)
 *  \brief Сложение элементов вектора с взвешенными элементами другого вектора для типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] w весовой коэффициент
 *  \param[out] dst выходные данные типа int32
 *  \param[in] size размер входных данных
 */
extern void w_vec32(int32_t *src0, int32_t *src1, int32_t w, int32_t *dst, int32_t size);
/*!
 *  \fn void w_vec_fl(float *src0, float *src1, float w, float *dst, int32_t size)
 *  \brief Сложение элементов вектора с взвешенными элементами другого вектора для типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] src1 входные данные типа float
 *  \param[in] w весовой коэффициент
 *  \param[out] dst выходные данные типа float
 *  \param[in] size размер входных данных
 */
extern void w_vec_fl(float *src0, float *src1, float w, float *dst, int32_t size);
/*!
 *  \fn void w_vec_db(double *src0, double *src1, double w, double *dst, int32_t size)
 *  \brief Сложение элементов вектора с взвешенными элементами другого вектора для типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] src1 входные данные типа double
 *  \param[in] w весовой коэффициент
 *  \param[out] dst выходные данные типа double
 *  \param[in] size размер входных данных
 */
extern void w_vec_db(double *src0, double *src1, double w, double *dst, int32_t size);

/*!
 *  \fn int32_t dotp_sqr(int32_t G, int16_t *src0, int16_t *src1, int32_t *r, int32_t size)
 *  \brief Подсчет суммы квадратов элементов второго вектора, вычисление скалярного произведения векторов типа int16
 *  \param[in] G коэффициент, который суммируется с результатом суммы квадратов
 *  \param[in] src0 входные данные типа int16
 *  \param[in] src1 входные данные типа int16
 *  \param[in] r скалярное произведение векторов
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32_t сумма квадратов элементов второго вектора
 */
extern int32_t dotp_sqr(int32_t G, int16_t *src0, int16_t *src1, int32_t *r, int32_t size);
/*!
 *  \fn int64_t dotp_sqr32(int64_t G, int32_t *src0, int32_t *src1, int64_t *r, int32_t size)
 *  \brief Подсчет суммы квадратов элементов второго вектора, вычисление скалярного произведения векторов типа int32
 *  \param[in] G коэффициент, который суммируется с результатом суммы квадратов
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] r скалярное произведение векторов
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int64_t сумма квадратов элементов второго вектора
 */
extern int64_t dotp_sqr32(int64_t G, int32_t *src0, int32_t *src1, int64_t *r, int32_t size);
/*!
 *  \fn float dotp_sqr_fl(float G, float *src0, float *src1, float *r, int32_t size)
 *  \brief Подсчет суммы квадратов элементов второго вектора, вычисление скалярного произведения векторов типа float
 *  \param[in] G коэффициент, который суммируется с результатом суммы квадратов
 *  \param[in] src0 входные данные типа float
 *  \param[in] src1 входные данные типа float
 *  \param[in] r скалярное произведение векторов
 *  \param[in] size размер входных данных
 *  \return
 *      - \b float сумма квадратов элементов второго вектора
 */
extern float dotp_sqr_fl(float G, float *src0, float *src1, float *r, int32_t size);
/*!
 *  \fn double dotp_sqr_db(double G, double *src0, double *src1, double *r, int32_t size)
 *  \brief Подсчет суммы квадратов элементов второго вектора, вычисление скалярного произведения векторов типа double
 *  \param[in] G коэффициент, который суммируется с результатом суммы квадратов
 *  \param[in] src0 входные данные типа double
 *  \param[in] src1 входные данные типа double
 *  \param[in] r скалярное произведение векторов
 *  \param[in] size размер входных данных
 *  \return
 *      - \b double сумма квадратов элементов второго вектора
 */
extern double dotp_sqr_db(double G, double *src0, double *src1, double *r, int32_t size);

/*!
 *  \fn int16_t maxval(int16_t *src0, int32_t size)
 *  \brief Поиск значения максимального элемента в векторе типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int16 значение максимума
 */
extern int16_t maxval(int16_t *src0, int32_t size);
/*!
 *  \fn int32_t maxval32(int32_t *src0, int32_t size)
 *  \brief Поиск значения максимального элемента в векторе типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32 значение максимума
 */
extern int32_t maxval32(int32_t *src0, int32_t size);
/*!
 *  \fn float maxval_fl(float *src0, int32_t size)
 *  \brief Поиск значения максимального элемента в векторе типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] size размер входных данных
 *  \return
 *      - \b float значение максимума
 */
extern float maxval_fl(float *src0, int32_t size);
/*!
 *  \fn double maxval_db(double *src0, int32_t size)
 *  \brief Поиск значения максимального элемента в векторе типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] size размер входных данных
 *  \return
 *      - \b double значение максимума
 */
extern double maxval_db(double *src0, int32_t size);

/*!
 *  \fn int16_t minval(int16_t *src0, int32_t size)
 *  \brief Поиск значения минимального элемента в векторе типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32 значение максимума
 */
extern int16_t minval(int16_t *src0, int32_t size);
/*!
 *  \fn int32_t minval32(int32_t *src0, int32_t size)
 *  \brief Поиск значения минимального элемента в векторе типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] size размер входных данных
 *  \return
 *      - \b int32 значение максимума
 */
extern int32_t minval32(int32_t *src0, int32_t size);
/*!
 *  \fn float minval_fl(float *src0, int32_t size)
 *  \brief Поиск значения минимального элемента в векторе типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] size размер входных данных
 *  \return
 *      - \b float значение максимума
 */
extern float minval_fl(float *src0, int32_t size);
/*!
 *  \fn double minval_db(double *src0, int32_t size)
 *  \brief Поиск значения минимального элемента в векторе типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] size размер входных данных
 *  \return
 *      - \b double значение максимума
 */
extern double minval_db(double *src0, int32_t size);

/*!
 *  \fn void mat_trans_scalar(int16_t *src0, int32_t rows, int32_t columns, int16_t *dst)
 *  \brief Транспонирование матрицы типы int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] rows количество строк матрицы
 *  \param[in] columns количество столбцов матрицы
 *  \param[out] dst выходные данные типа int16
 */
extern void mat_trans_scalar(int16_t *src0, int32_t rows, int32_t columns, int16_t *dst);
/*!
 *  \fn void mat_trans_scalar_fl(float *src0, int32_t rows, int32_t columns, float *dst)
 *  \brief Транспонирование матрицы типы float
 *  \param[in] src0 входные данные типа float
 *  \param[in] rows количество строк матрицы
 *  \param[in] columns количество столбцов матрицы
 *  \param[out] dst выходные данные типа float
 */
extern void mat_trans_scalar_fl(float *src0, int32_t rows, int32_t columns, float *dst);
/*!
 *  \fn void mat_trans_scalar_s32(int32_t *src0, int32_t rows, int32_t columns, int32_t *dst)
 *  \brief Транспонирование матрицы типы int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] rows количество строк матрицы
 *  \param[in] columns количество столбцов матрицы
 *  \param[out] dst выходные данные типа int32
 */
extern void mat_trans_scalar_s32(int32_t *src0, int32_t rows, int32_t columns, int32_t *dst);
/*!
 *  \fn void mat_trans_scalar_db(double *src0, int32_t rows, int32_t columns, double *dst)
 *  \brief Транспонирование матрицы типы double
 *  \param[in] src0 входные данные типа double
 *  \param[in] rows количество строк матрицы
 *  \param[in] columns количество столбцов матрицы
 *  \param[out] dst выходные данные типа double
 */
extern void mat_trans_scalar_db(double *src0, int32_t rows, int32_t columns, double *dst);

/*!
 *  \fn void mat_mul(int16_t *src0, int32_t rows0, int32_t columns0, int16_t *src1, int32_t columns1, int16_t *dst, int32_t shift)
 *  \brief Умножение матриц типа int16
 *  \param[in] src0 входные данные типа int16
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа int16
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа int16
 *  \param[in] shift сдвиг результата
 */
extern void mat_mul(int16_t *src0, int32_t rows0, int32_t columns0, int16_t *src1, int32_t columns1, int16_t *dst, int32_t shift);
/*!
 *  \fn void mat_mul32(int32_t *src0, int32_t rows0, int32_t columns0, int32_t *src1, int32_t columns1, int32_t *dst, int32_t shift)
 *  \brief Умножение матриц типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа int32
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа int32
 *  \param[in] shift сдвиг результата
 */
extern void mat_mul32(int32_t *src0, int32_t rows0, int32_t columns0, int32_t *src1, int32_t columns1, int32_t *dst, int32_t shift);
/*!
 *  \fn void mat_mul_fl(float *src0, int32_t rows0, int32_t columns0, float *src1, int32_t columns1, float *dst)
 *  \brief Умножение матриц типа float
 *  \param[in] src0 входные данные типа float
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа float
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа float
 */
extern void mat_mul_fl(float *src0, int32_t rows0, int32_t columns0, float *src1, int32_t columns1, float *dst);
/*!
 *  \fn void mat_mul_db(double *src0, int32_t rows0, int32_t columns0, double *src1, int32_t columns1, double *dst)
 *  \brief Умножение матриц типа double
 *  \param[in] src0 входные данные типа double
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа double
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа double
 */
extern void mat_mul_db(double *src0, int32_t rows0, int32_t columns0, double *src1, int32_t columns1, double *dst);
/*!
 *  \fn void mat_mul_cplx(int16_t *src0, int32_t rows0, int32_t columns0, int16_t *src1, int32_t columns1, int16_t *dst, int32_t shift)
 *  \brief Комплексное умножение матриц типа int16 (cint16)
 *  \param[in] src0 входные данные типа int16 (cint16)
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа int16 (cint16)
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа int16 (cint16) референсной функции
 *  \param[in] shift сдвиг результата
 */
extern void mat_mul_cplx(int16_t *src0, int32_t rows0, int32_t columns0, int16_t *src1, int32_t columns1, int16_t *dst, int32_t shift);
/*!
 *  \fn void mat_mul_cplx32(int32_t *src0, int32_t rows0, int32_t columns0, int32_t *src1, int32_t columns1, int32_t *dst, int32_t shift)
 *  \brief Комплексное умножение матриц типа int32 (cint32)
 *  \param[in] src0 входные данные типа int32 (cint32)
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа int32 (cint32)
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа int32 (cint32) референсной функции
 *  \param[in] shift сдвиг результата
 */
extern void mat_mul_cplx32(int32_t *src0, int32_t rows0, int32_t columns0, int32_t *src1, int32_t columns1, int32_t *dst, int32_t shift);
/*!
 *  \fn void mat_mul_cplx_fl(float *src0, int32_t rows0, int32_t columns0, float *src1, int32_t columns1, float *dst)
 *  \brief Комплексное умножение матриц типа float (cfloat)
 *  \param[in] src0 входные данные типа float (cfloat)
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа float (cfloat)
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа float (cfloat) референсной функции
 */
extern void mat_mul_cplx_fl(float *src0, int32_t rows0, int32_t columns0, float *src1, int32_t columns1, float *dst);
/*!
 *  \fn void mat_mul_cplx_db(double *src0, int32_t rows0, int32_t columns0, double *src1, int32_t columns1, double *dst)
 *  \brief Комплексное умножение матриц типа double (cdouble)
 *  \param[in] src0 входные данные типа double (cdouble)
 *  \param[in] rows0 количество строк первой матрицы
 *  \param[in] columns0 количество столбцов первой матрицы
 *  \param[in] src1 входные данные типа double (cdouble)
 *  \param[in] columns1 количество столбцов второй матрицы
 *  \param[out] dst выходные данные типа double (cdouble) референсной функции
 */
extern void mat_mul_cplx_db(double *src0, int32_t rows0, int32_t columns0, double *src1, int32_t columns1, double *dst);
/*!
 *  \fn void recip16(int16_t *x, int16_t *rfrac, int16_t* rexp, int32_t size)
 *  \brief Обратная величина для чисел с фиксированной точкой типа int16
 *  \param[in] x входные данные типа int16
 *  \param[out] rfrac выходные данные типа int16, нормализованное значение
 *  \param[out] rexp выходные данные типа int16, значение степени экспоненты
 *  \param[in] size размер входных массивов
 */
extern void recip16(int16_t *x, int16_t *rfrac, int16_t* rexp, int32_t size);

/// Для операций с фиксированной точкой числа записываются в int32,
/// Младшие 16 бит - дробная часть, старшие - целая.
/// Порядок следования байтов little-endian.

/// Умножение чисел с фиксированной точкой (i=16,f=16)
extern void mul16i16f(
    int32_t *src0,                  ///< [in]  входные данные первого массива
    int32_t *src1,                  ///< [in]  входные данные второго массива
    int32_t *dst,                   ///< [out] выходные данные массива
    int32_t size                    ///< [in]  количество элементов входных массивов
);

/*!
 *  \fn void add_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst)
 *  \brief Сложение комплексных чисел
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[out] dst выходные данные типа int32
 */
extern void add_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst);

/*!
 *  \fn void acc_cplx32(int32_t *dst, int32_t *src0, int32_t size)
 *  \brief Комплексное векторное сложение с накоплением
 *  \param[in/out] dst входные/выходные данные типа int32
 *  \param[in] src0 входные данные типа int32
 *  \param[in] size размер входных массивов
 */
extern void acc_cplx32(int32_t *dst, int32_t *src0, int32_t size);

/*!
 *  \fn void sub_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst)
 *  \brief Комплексное векторное вычитание
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[out] dst выходные данные типа int32
 */
extern void sub_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst);

/*!
 *  \fn void mul_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst)
 *  \brief Комплексное векторное умножение
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[out] dst выходные данные типа int32
 */
extern void mul_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst);


/// Комплексное векторное умножение порядок следования Re, Im для типа int16_t
extern void mul_cplx16_re_im(
    int16_t *src0,              ///< [in]  входные данные для первого массива
    int16_t *src1,              ///< [in]  входные данные для второго массива
    int32_t size,               ///< [in]  количество комплексных элементов входных массивов
    int16_t *dst                ///< [out] выходные данные
);

/// Комплексное векторное умножение порядок следования Re, Im для типа int32_t
extern void mul_cplx32_re_im(
    int32_t *src0,              ///< [in]  входные данные для первого массива
    int32_t *src1,              ///< [in]  входные данные для второго массива
    int32_t size,               ///< [in]  количество комплексных элементов входных массивов
    int32_t *dst                ///< [out] выходные данные
);

/// Комплексное векторное умножение порядок следования Re, Im для типа float
extern void mul_cplx_fl_re_im(
    float *src0,                ///< [in]  входные данные для первого массива
    float *src1,                ///< [in]  входные данные для второго массива
    int32_t size,               ///< [in]  количество комплексных элементов входных массивов
    float *dst                  ///< [out] выходные данные
);

/// Комплексное векторное умножение порядок следования Re, Im для типа double
extern void mul_cplx_db_re_im(
    double *src0,               ///< [in]  входные данные для первого массива
    double *src1,               ///< [in]  входные данные для второго массива
    int32_t size,               ///< [in]  количество комплексных элементов входных массивов
    double *dst                 ///< [out] выходные данные
);

/*!
 *  \fn void mul_conj_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst)
 *  \brief Комплексное векторное умножение c комплексным сопряжением второго вектора
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[out] dst выходные данные типа int32
 */
extern void mul_conj_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t *dst);

/*!
 *  \fn void add_rconst_cplx32(int32_t *dst, int32_t size, int32_t value)
 *  \brief Добавление числа к каждому элементу вектора
 *  \param[in/out] dst входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] value коэффициент типа int32
 */
extern void add_rconst_cplx32(int32_t *dst, int32_t size, int32_t value);

/*!
 *  \fn void mul_rconst_cplx32(int32_t *dst, int32_t size, int32_t value)
 *  \brief Умножение вектора на реальное число
 *  \param[in/out] dst входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] value коэффициент типа int32
 */
extern void mul_rconst_cplx32(int32_t *dst, int32_t size, int32_t value);

/*!
 *  \fn void scale_cplx32(int32_t *dst, int32_t size, int32_t value)
 *  \brief Масштабирование вектора на степень двойки
 *  \param[in/out] dst входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] value коэффициент типа int32
 */
extern void scale_cplx32(int32_t *dst, int32_t size, int32_t value);

/*!
 *  \fn void div_rconst_cplx32(int32_t *dst, int32_t size, int32_t value)
 *  \brief Деление вектора на реальное число
 *  \param[in/out] dst входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] value коэффициент типа int32
 */
extern void div_rconst_cplx32(int32_t *dst, int32_t size, int32_t value);

/*!
 *  \fn void sum_cplx32(int32_t *dst, int32_t size, int32_t* res)
 *  \brief Сумма всех элементов вектора
 *  \param[in/out] dst входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] res указатель на результат int32
 */
extern void sum_cplx32(int32_t *dst, int32_t size, int32_t* res);

/*!
 *  \fn void abs_cplx32(int32_t *src, int32_t size, int32_t* res)
 *  \brief Расчет модуля
 *  \param[in] src входные данные типа int32
 *  \param[in] size размер входных массивов
 *  \param[in] res указатель на результат int32
 */
extern void abs_cplx32(int32_t *src, int32_t size, int32_t* res);

/*!
 *  \fn void conv_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t size_ker, int32_t *dst)
 *  \brief Расчет свертки
 *  \param[in] src0 входные данные типа int32
 *  \param[in] src1 входные данные ядра типа int32
 *  \param[in] size размер входного массива
 *  \param[in] size_ker размер ядра свертки
 *  \param[in] dst указатель на результат int32
 */
extern void conv_cplx32(int32_t *src0, int32_t *src1, int32_t size, int32_t size_ker, int32_t *dst);

/// Поэлементное перемножение двух массивов типа int16
extern void muls16(
    const int16_t *src0,            ///< [in]  входные данные первого массива
    const int16_t *src1,            ///< [in]  входные данные второго массива
    int16_t *dst,                   ///< [out] выходные данные массива
    const int32_t size              ///< [in]  количество элементов выходного массива
);

/// Поэлементное перемножение двух массивов типа int32
extern void muls32(
    const int32_t *src0,            ///< [in]  входные данные первого массива
    const int32_t *src1,            ///< [in]  входные данные второго массива
    int32_t *dst,                   ///< [out] выходные данные массива
    const int32_t size              ///< [in]  количество элементов выходного массива
);

/// Поэлементное перемножение двух массивов типа float
extern void mul_fl(
    const float *src0,              ///< [in]  входные данные первого массива
    const float *src1,              ///< [in]  входные данные второго массива
    float *dst,                     ///< [out] выходные данные массива
    const int32_t size              ///< [in]  количество элементов выходного массива
);

/// Поэлементное перемножение двух массивов типа double
extern void mul_db(
    const double *src0,             ///< [in]  входные данные первого массива
    const double *src1,             ///< [in]  входные данные второго массива
    double *dst,                    ///< [out] выходные данные массива
    const int32_t size              ///< [in]  количество элементов выходного массива
);

#endif
