/*! \file
 *  \brief Заголовочный файл с референсными функциями библиотеки
 */

#ifndef REFERENCE_H
#define REFERENCE_H

#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

/*!
 *  \fn void fft_mod2(const float *x, float *y, const int N,  const bool direct)
 *  \brief БПФ для типа float
 *  \param[in] x входные данные типа float
 *  \param[out] y выходные данные типа float
 *  \param[in] N размер преобразования
 *  \param[in] direct флаг для обратного преобразования
 */
void fft_mod2(const float *x, float *y, const int N,  const bool direct);
/*!
 *  \fn int reverse_b(int val, const int logN)
 *  \brief Изменение порядка бит в номере элемента массива
 *  \param[in] val индекс элемента массива
 *  \param[in] logN log2 от размера массива
 *  \return
 *      - \b int новый номер элемента
 */
int reverse_b(int val, const int logN);
/*!
 *  \fn void dft(float *x, float *y, const int N)
 *  \brief Преобразование Фурье для типа float
 *  \param[in] x входные данные типа float
 *  \param[out] y выходные данные типа float
 *  \param[in] N размер преобразования
 */
void dft(float *x, float *y, const int N);

#endif
